import { Role } from '@modules/ad/core/domain/ad.types';
import { Waypoint } from '@modules/ad/core/domain/candidate.types';
import {
  Path,
  PathCreator,
  PathType,
} from '@modules/ad/core/domain/patch-creator.service';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
};
const intermediateWaypoint: Waypoint = {
  position: 1,
  lat: 48.74488,
  lon: 4.8972,
};

describe('Path Creator Service', () => {
  it('should create a path for a driver only', () => {
    const pathCreator: PathCreator = new PathCreator(
      [Role.DRIVER],
      [originWaypoint, destinationWaypoint],
    );
    const paths: Path[] = pathCreator.getPaths();
    expect(paths).toHaveLength(1);
    expect(paths[0].type).toBe(PathType.DRIVER);
  });
  it('should create a path for a passenger only', () => {
    const pathCreator: PathCreator = new PathCreator(
      [Role.PASSENGER],
      [originWaypoint, destinationWaypoint],
    );
    const paths: Path[] = pathCreator.getPaths();
    expect(paths).toHaveLength(1);
    expect(paths[0].type).toBe(PathType.PASSENGER);
  });
  it('should create a single path for a driver and passenger', () => {
    const pathCreator: PathCreator = new PathCreator(
      [Role.DRIVER, Role.PASSENGER],
      [originWaypoint, destinationWaypoint],
    );
    const paths: Path[] = pathCreator.getPaths();
    expect(paths).toHaveLength(1);
    expect(paths[0].type).toBe(PathType.GENERIC);
  });
  it('should create two different paths for a driver and passenger with intermediate waypoint', () => {
    const pathCreator: PathCreator = new PathCreator(
      [Role.DRIVER, Role.PASSENGER],
      [
        originWaypoint,
        intermediateWaypoint,
        { ...destinationWaypoint, position: 2 },
      ],
    );
    const paths: Path[] = pathCreator.getPaths();
    expect(paths).toHaveLength(2);
    expect(
      paths.filter((path: Path) => path.type == PathType.DRIVER),
    ).toHaveLength(1);
    expect(
      paths.filter((path: Path) => path.type == PathType.DRIVER)[0].waypoints,
    ).toHaveLength(3);
    expect(
      paths.filter((path: Path) => path.type == PathType.PASSENGER),
    ).toHaveLength(1);
    expect(
      paths.filter((path: Path) => path.type == PathType.PASSENGER)[0]
        .waypoints,
    ).toHaveLength(2);
  });
});
